#! /usr/bin/env python

""" This is the event system. It is distinct from pygame's event
system, because this one is truly and entirely callback-driven.
"""

import weakref, new
from weakref import  ReferenceError
from exceptions import KeyError, NotImplementedError

###############################################################################################


class Proxy(object):
  '''
  Our own proxy object which enables weak references to bound and unbound
  methods and arbitrary callables. Pulls information about the function,
  class, and instance out of a bound method. Stores a weak reference to the
  instance to support garbage collection.

  @organization: IBM Corporation
  @copyright: Copyright (c) 2005, 2006 IBM Corporation
  @license: The BSD License
  '''
  def __object_died (self, obj):
      if self.__callback:
          self.__callback (self)
  
  def __init__(self, cb, callback=None):
      self.__callback = callback
      try:
          try:
              self.inst = weakref.ref(cb.im_self, self.__object_died)
          except TypeError:
              self.inst = None
          self.func = cb.im_func
          self.klass = cb.im_class
      except AttributeError:
          self.inst = None
      try:
          self.func = cb.im_func
      except:
          self.func = weakref.proxy(cb, self.__object_died)
      self.klass = None

  def __call__(self, *args, **kwargs):
    '''
    Proxy for a call to the weak referenced object. Take arbitrary params to
    pass to the callable.

    @raise ReferenceError: When the weak reference refers to a dead object
    '''
    if self.inst is not None and self.inst() is None:
      raise ReferenceError
    elif self.inst is not None:
      # build a new instance method with a strong reference to the instance
      mtd = new.instancemethod(self.func, self.inst(), self.klass)
    else:
      # not a bound method, just return the func
      mtd = self.func
    # invoke the callable and return the result
    return mtd(*args, **kwargs)

  def __eq__(self, other):
    '''
    Compare the held function and instance with that held by another proxy.

    @param other: Another proxy object
    @type other: L{Proxy}
    @return: Whether this func/inst pair is equal to the one in the other
    proxy object or not
    @rtype: boolean
    '''
    try:
      return self.func == other.func and self.inst() == other.inst()
    except Exception:
      pass
    try:
      return self.func == other.func
    except Exception:
      pass
    return False

  def __ne__(self, other):
    '''
    Inverse of __eq__.
    '''
    return not self.__eq__(other)

  def __repr__ (self):
      return "Proxy for function %s (instance %s)" % (self.func, self.inst)

    
    
###########################################################################################    
#                                       Event                                             #    
###########################################################################################


def Event ():
    """ A simple function that builds another function that will act as an event.
    
    The principle of it is to use some python refence couting magic :
    Event () is B{just a function}. It declares an empty list that will serve as the
    callback holder, and three functions : register, unregister and event_caller.
    
    register and unregister are added as attributes of event_caller, who is
    returned at the end of the function. The trick is that those functions all use
    callback_list as a local variable : thanks to reference counting, callback_list
    is not deleted as long as those three functions are alive !
    
    register and unregister both take one argument : a callback to add or remove
    from the list.
    
    When the event is called, it calls all the callbacks in callback_list with THE
    EXACT SAME ARGUMENTS !
    
    See the code for more understanding on how it works (it's actually pretty simple).
    """
    
    callback_list = []
    
    def register (callback):
        callback_proxy = Proxy (callback)
        callback_list.append (callback_proxy)
        
    def unregister (callback):
        callback_proxy = Proxy (callback)
        try:
            callback_list.remove (callback_proxy)
        except:
            pass
    
    def event_caller (*args, **kwargs):
        for f in callback_list:
            f (*args, **kwargs)
                    
    event_caller.register = register
    event_caller.unregister = unregister
    event_caller.__name__ = "EventCaller"
    return event_caller

